from presidio_analyzer import AnalyzerEngine


def parse_text(text: str) -> tuple[str, dict]:
    anonymizer_dict = {}
    entity_count_dict = {}
    processed_entities = []

    analyzer = AnalyzerEngine()
    # replace "'" with space to be able to detect names, will revert back when final output
    transformed_text = text.replace("'", " ")
    results = analyzer.analyze(text=transformed_text, language="en", score_threshold=0.4)

    # count the number of entity type occurrence for each type
    for entity in results:

        entity_name = text[entity.start: entity.end]
        if entity.entity_type in entity_count_dict:
            if entity_name in processed_entities:
                continue
            else:
                entity_count_dict[entity.entity_type] += 1
        else:
            entity_count_dict[entity.entity_type] = 1

        processed_entities.append(entity_name)
        new_entity_type = f'{entity.entity_type}{entity_count_dict[entity.entity_type]}'
        anonymizer_dict[entity_name] = f'<{new_entity_type}>'

    for key, value in anonymizer_dict.items():
        text = text.replace(key, value)

    # return cleaned text and anonymizer mapping
    return text, {v: k for k, v in anonymizer_dict.items()}

# print(parse_text(text="Aria' el Bright and Ismael are trying his best to do it. But Aria' el Bright is not that good!"))



from langchain_community.chat_models import ChatOllama
from src.chain_guardrail.main import ChainValidator

model = ChatOllama(
    base_url='http://localhost:11434',
    model="llama3",
    temperature=0,
)
validator_obj = ChainValidator()

chain = model | validator_obj.static_validator
print(chain.invoke("give me two examples of person's name"))



from langchain_core.messages import AIMessage

from chain_guardrail.pii.presidio.parser import parse_text


class ChainValidator:
    def __init__(self):
        self.anonymizer_dict = {}

    def static_validator(self, ai_message: AIMessage) -> str:
        """Parse the AI message."""
        filtered_text, anonymizer_dict = parse_text(ai_message.content)
        self.anonymizer_dict = anonymizer_dict
        print(f'Anonymizer dict:\n {anonymizer_dict}')
        return filtered_text
